#include "arduino_secrets.h"
/*
  Sketch generated by the Arduino IoT Cloud Thing "Untitled"
  https://create.arduino.cc/cloud/things/954d8895-32a9-4c2f-904a-624dae11a06f

  Arduino IoT Cloud Variables description

  The following variables are automatically generated and updated when changes are made to the Thing

  CloudSchedule closingSchedule;
  CloudSchedule openingSchedule;
  CloudTime timeRead;

  Variables which are marked as READ/WRITE in the Cloud Thing will also have functions
  which are called when their values are changed from the Dashboard.
  These functions are generated with the Thing and added at the end of this sketch.
*/

#include "thingProperties.h"

// !!!!!! Define hardware pins
// Motor Controller
#define EN_PIN 7  // Enabled
#define DIR_PIN 6 // Step
#define PUL_PIN 5 // Direction

// Limit Switch
#define LIM_PIN 12

// Variables
const int MOTOR_SPEED = 2000;
const bool UP_DIR = 1;
const bool DOWN_DIR = 0;

bool running_open;
bool running_close;
bool at_top;

void setup()
{
  // Initialize serial and wait for port to open:
  Serial.begin(9600);
  // This delay gives the chance to wait for a Serial Monitor without blocking if none is found
  delay(1500);

  // Defined in thingProperties.h
  initProperties();

  // Connect to Arduino IoT Cloud
  ArduinoCloud.begin(ArduinoIoTPreferredConnection);

  /*
     The following function allows you to obtain more information
     related to the state of network and IoT Cloud connection and errors
     the higher number the more granular information you’ll get.
     The default is 0 (only errors).
     Maximum is 4
 */
  setDebugMessageLevel(2);
  ArduinoCloud.printDebugInfo();

  // Set pin modes, initial values
  pinMode(EN_PIN, OUTPUT);   // set the EN_PIN as an output
  digitalWrite(EN_PIN, LOW); // set the enabled pin to low (enabled)

  pinMode(DIR_PIN, OUTPUT);   // set the DIR_PIN as an output
  digitalWrite(DIR_PIN, LOW); // set the direction pin to low

  pinMode(PUL_PIN, OUTPUT);   // set the STEP_PIN as an output
  digitalWrite(PUL_PIN, LOW); // set the step pin to low

  pinMode(LIM_PIN, INPUT_PULLUP);

  Serial.print("Hello world!");
}

// This function sets the number of steps, the direction and the speed
// steps: a full rotation requires 1600 steps
// direction: 1 = clockwise, 0 = anticlockwise
// speed: number of microseconds between each step, min 100
void rotateMotor(int steps, bool direction, int speed)
{

  // Serial.println("Rotating Motor");

  // Set the motor direction
  digitalWrite(DIR_PIN, direction);

  // Step the motor
  for (int i = 0; i < steps; i++)
  {
    digitalWrite(PUL_PIN, HIGH);
    delayMicroseconds(speed);
    digitalWrite(PUL_PIN, LOW);
    delayMicroseconds(speed);
  }
}

// Wrapper to make enabling/disabling the motor controller more intuitive
void setMotorEnabled(bool enabled)
{
  // Since writing LOW = enabled, switch input
  digitalWrite(EN_PIN, not enabled);
}

/*  Run motor until hit limit switch, so we know we are at the top of the blinds.
 *  Then, lower slightly s.t. we are not activating limit switch anymore.
 */
void raiseBlinds()
{
  Serial.println("Raising blind.");
  setMotorEnabled(true);

  Serial.println("Move up!");
  while (not at_top)
  {
    rotateMotor(1, UP_DIR, MOTOR_SPEED);

    // When limit switch has been hit, it reads LOW
    at_top = not digitalRead(LIM_PIN);
  }
  Serial.println("Hit limit switch!");

  Serial.println("Move down!");
  // Don't want to constantly be hitting the limit switch!
  rotateMotor(200, DOWN_DIR, MOTOR_SPEED);
  at_top = false;

  Serial.println("Done raising.");
  setMotorEnabled(false);
}

void lowerBlinds()
{
  Serial.println("Lower blinds.");

  // Raise to top, to calibrate
  Serial.println("Calibrate blinds.");
  raiseBlinds();

  setMotorEnabled(true);
  // Lower by set amount
  Serial.println("Move down!");
  rotateMotor(160, DOWN_DIR, MOTOR_SPEED);

  Serial.println("Done lowering.");
  setMotorEnabled(false);
}

void loop()
{
  ArduinoCloud.update();
  // Your code here

  if (openingSchedule.isActive())
  {
    if (not running_open)
    {
      Serial.println("Scheduled opening blinds!");
      running_open = true;
      raiseBlinds();
    }
  }
  else
  {
    running_open = false;
  }

  if (closingSchedule.isActive())
  {
    if (not running_close)
    {
      Serial.println("Scheduled closing blinds!");
      running_close = true;
      lowerBlinds();
    }
  }
  else
  {
    running_close = false;
  }

  if (ArduinoCloud.connected())
  {
    timeRead = ArduinoCloud.getLocalTime();
  }
}
